=== modified file 'drizzled/db.cc'
--- drizzled/db.cc	2009-01-26 09:55:04 +0000
+++ drizzled/db.cc	2009-02-02 04:55:56 +0000
@@ -166,9 +166,106 @@
     (collation_server).
   */
 
+  load_db_opt_by_name(session, db_name, &db_info);
+
   return db_info.default_table_charset;
 }
 
+/* path is path to database, not schema file */
+static int write_schema_file(Session *session,
+			     const char *path, const char *name,
+			     HA_CREATE_INFO *create)
+{
+  drizzle::Schema db;
+  char schema_file_tmp[FN_REFLEN];
+  string schema_file(path);
+
+  assert(path);
+  assert(name);
+  assert(create);
+
+  snprintf(schema_file_tmp, FN_REFLEN, "%s%cdb.opt.tmpXXXXXX",path,FN_LIBCHAR);
+
+  schema_file.append(1, FN_LIBCHAR);
+  schema_file.append("db.opt");
+
+  int fd= mkstemp(schema_file_tmp);
+
+  if (fd==-1)
+    return errno;
+
+  if (!create->default_table_charset)
+    create->default_table_charset= session->variables.collation_server;
+
+  db.set_name(name);
+  db.set_collation(create->default_table_charset->name);
+
+  if (!db.SerializeToFileDescriptor(fd))
+  {
+    close(fd);
+    unlink(schema_file_tmp);
+    return -1;
+  }
+
+  if (rename(schema_file_tmp, schema_file.c_str()) == -1)
+  {
+    close(fd);
+    return errno;
+  }
+
+  close(fd);
+  return 0;
+}
+
+int load_db_opt(Session *session, const char *path, HA_CREATE_INFO *create)
+{
+  drizzle::Schema db;
+  string buffer;
+
+  memset(create, 0, sizeof(*create));
+  create->default_table_charset= session->variables.collation_server;
+
+  int fd= open(path, O_RDONLY);
+
+  if (fd == -1)
+    return errno;
+
+  if (!db.ParseFromFileDescriptor(fd))
+  {
+    close(fd);
+    return -1;
+  }
+  close(fd);
+
+  buffer= db.collation();
+  if (!(create->default_table_charset= get_charset_by_name(buffer.c_str(),
+							   MYF(0))))
+  {
+    errmsg_printf(ERRMSG_LVL_ERROR,
+		  _("Error while loading database options: '%s':"),path);
+    errmsg_printf(ERRMSG_LVL_ERROR, ER(ER_UNKNOWN_COLLATION), buffer.c_str());
+    create->default_table_charset= default_charset_info;
+    return -1;
+  }
+
+  return 0;
+}
+
+int load_db_opt_by_name(Session *session, const char *db_name,
+			HA_CREATE_INFO *db_create_info)
+{
+  char db_opt_path[FN_REFLEN];
+
+  /*
+    Pass an empty file name, and the database options file name as extension
+    to avoid table name to file name encoding.
+  */
+  (void) build_table_filename(db_opt_path, sizeof(db_opt_path),
+                              db_name, "", "db.opt", 0);
+
+  return load_db_opt(session, db_opt_path, db_create_info);
+}
+
 
 /*
   Create a database
@@ -200,7 +297,6 @@
   char	 tmp_query[FN_REFLEN+16];
   long result= 1;
   int error= 0;
-  struct stat stat_info;
   uint32_t create_options= create_info ? create_info->options : 0;
   uint32_t path_len;
 
@@ -235,31 +331,34 @@
   path_len= build_table_filename(path, sizeof(path), db, "", "", 0);
   path[path_len-1]= 0;                    // Remove last '/' from path
 
-  if (!stat(path,&stat_info))
+  if (mkdir(path,0777) == -1)
   {
-    if (!(create_options & HA_LEX_CREATE_IF_NOT_EXISTS))
+    if (errno == EEXIST)
     {
-      my_error(ER_DB_CREATE_EXISTS, MYF(0), db);
-      error= -1;
+      if (!(create_options & HA_LEX_CREATE_IF_NOT_EXISTS))
+      {
+	my_error(ER_DB_CREATE_EXISTS, MYF(0), db);
+	error= -1;
+	goto exit;
+      }
+      push_warning_printf(session, DRIZZLE_ERROR::WARN_LEVEL_NOTE,
+			  ER_DB_CREATE_EXISTS, ER(ER_DB_CREATE_EXISTS), db);
+      if (!silent)
+	my_ok(session);
+      error= 0;
       goto exit;
     }
-    push_warning_printf(session, DRIZZLE_ERROR::WARN_LEVEL_NOTE,
-			ER_DB_CREATE_EXISTS, ER(ER_DB_CREATE_EXISTS), db);
-    if (!silent)
-      my_ok(session);
-    error= 0;
+
+    my_error(ER_CANT_CREATE_DB, MYF(0), db, my_errno);
+    error= -1;
     goto exit;
   }
-  else
+
+  error= write_schema_file(session, path, db, create_info);
+  if (error && error != EEXIST)
   {
-    if (errno != ENOENT)
-    {
-      my_error(EE_STAT, MYF(0), path, errno);
-      goto exit;
-    }
-    if (mkdir(path,0777) < 0)
-    {
-      my_error(ER_CANT_CREATE_DB, MYF(0), db, my_errno);
+    if (rmdir(path) >= 0)
+    {
       error= -1;
       goto exit;
     }
@@ -298,7 +397,9 @@
 bool mysql_alter_db(Session *session, const char *db, HA_CREATE_INFO *create_info)
 {
   long result=1;
-  int error= false;
+  int error= 0;
+  char	 path[FN_REFLEN+16];
+  uint32_t path_len;
 
   /*
     Do not alter database if another thread is holding read lock.
@@ -318,6 +419,16 @@
   pthread_mutex_lock(&LOCK_drizzle_create_db);
 
   /* Change options if current database is being altered. */
+  path_len= build_table_filename(path, sizeof(path), db, "", "", 0);
+  path[path_len-1]= 0;                    // Remove last '/' from path
+
+  error= write_schema_file(session, path, db, create_info);
+  if (error && error != EEXIST)
+  {
+    /* TODO: find some witty way of getting back an error message */
+    pthread_mutex_unlock(&LOCK_drizzle_create_db);
+    goto exit;
+  }
 
   if (session->db && !strcmp(session->db,db))
   {
@@ -390,6 +501,8 @@
   pthread_mutex_lock(&LOCK_drizzle_create_db);
 
   length= build_table_filename(path, sizeof(path), db, "", "", 0);
+  strcpy(path+length, "db.opt");         // Append db option file name
+  unlink(path);
   path[length]= '\0';				// Remove file name
 
   /* See if the directory exists */

=== modified file 'drizzled/db.h'
--- drizzled/db.h	2009-01-26 09:55:04 +0000
+++ drizzled/db.h	2009-02-02 04:55:56 +0000
@@ -34,6 +34,8 @@
 
 
 bool check_db_dir_existence(const char *db_name);
+int load_db_opt_by_name(Session *session, const char *db_name,
+			HA_CREATE_INFO *db_create_info);
 const CHARSET_INFO *get_default_db_collation(Session *session, const char *db_name);
 
 extern int creating_database; // How many database locks are made

=== modified file 'drizzled/serialize/schema.proto'
--- drizzled/serialize/schema.proto	2008-08-13 23:12:06 +0000
+++ drizzled/serialize/schema.proto	2009-02-02 04:55:56 +0000
@@ -2,6 +2,5 @@
 
 message Schema {
   required string name = 1;
-  optional string characterset = 2;
-  optional string collation = 3;
+  optional string collation = 2;
 }

=== modified file 'drizzled/serialize/schema_reader.cc'
--- drizzled/serialize/schema_reader.cc	2008-12-16 03:09:04 +0000
+++ drizzled/serialize/schema_reader.cc	2009-02-02 04:55:56 +0000
@@ -13,8 +13,6 @@
   cout << "CREATE SCHEMA `" << schema->name() << "` ";
   if (schema->has_collation())
     cout << "COLLATE `" << schema->collation() << "` ";
-  if (schema->has_characterset())
-    cout << "CHARACTER SET `" << schema->characterset() <<"` ";
   cout << ";" << endl;
 }
 

=== modified file 'drizzled/show.cc'
--- drizzled/show.cc	2009-01-30 06:16:09 +0000
+++ drizzled/show.cc	2009-02-02 04:55:56 +0000
@@ -390,7 +390,7 @@
   String buffer(buff, sizeof(buff), system_charset_info);
   Protocol *protocol=session->protocol;
 
-  if (store_db_create_info(dbname, &buffer, create_info))
+  if (store_db_create_info(session, dbname, &buffer, create_info))
   {
     /*
       This assumes that the only reason for which store_db_create_info()
@@ -925,7 +925,7 @@
   @returns true if errors are detected, false otherwise.
 */
 
-bool store_db_create_info(const char *dbname, String *buffer,
+bool store_db_create_info(Session *session, const char *dbname, String *buffer,
                           HA_CREATE_INFO *create_info)
 {
   HA_CREATE_INFO create;
@@ -941,6 +941,8 @@
   {
     if (check_db_dir_existence(dbname))
       return(true);
+
+    load_db_opt_by_name(session, dbname, &create);
   }
 
   buffer->length(0);
@@ -2679,8 +2681,11 @@
       continue;
     }
     {
+      HA_CREATE_INFO create;
+      load_db_opt_by_name(session, db_name->str, &create);
+
       if (store_schema_shemata(session, table, db_name,
-                               system_charset_info))
+                               create.default_table_charset))
         return(1);
     }
   }

=== modified file 'drizzled/show.h'
--- drizzled/show.h	2009-01-26 09:55:04 +0000
+++ drizzled/show.h	2009-02-02 04:55:56 +0000
@@ -59,7 +59,7 @@
 
 int store_create_info(Session *session, TableList *table_list, String *packet,
                       HA_CREATE_INFO  *create_info_arg);
-bool store_db_create_info(const char *dbname, String *buffer,
+bool store_db_create_info(Session *session, const char *dbname, String *buffer,
                           HA_CREATE_INFO *create_info);
 bool schema_table_store_record(Session *session, Table *table);
 

=== modified file 'drizzled/sql_table.cc'
--- drizzled/sql_table.cc	2009-01-30 06:16:09 +0000
+++ drizzled/sql_table.cc	2009-02-02 04:55:56 +0000
@@ -78,6 +78,24 @@
                           HA_CREATE_INFO *create_info,
                           Alter_info *alter_info);
 
+static void set_table_default_charset(Session *session,
+                                      HA_CREATE_INFO *create_info, char *db)
+{
+  /*
+    If the table character set was not given explicitly,
+    let's fetch the database default character set and
+    apply it to the table.
+  */
+  if (!create_info->default_table_charset)
+  {
+    HA_CREATE_INFO db_info;
+
+    load_db_opt_by_name(session, db, &db_info);
+
+    create_info->default_table_charset= db_info.default_table_charset;
+  }
+}
+
 /*
   Translate a file name to a table name (WL #1324).
 
@@ -1746,10 +1764,7 @@
     return(true);
   }
 
-  if (!create_info->default_table_charset)
-  {
-    create_info->default_table_charset= system_charset_info;
-  }
+  set_table_default_charset(session, create_info, (char*) db);
 
   if (mysql_prepare_create_table(session, create_info, alter_info,
                                  internal_tmp_table,
@@ -4651,10 +4666,7 @@
   if (mysql_prepare_alter_table(session, table, create_info, alter_info))
       goto err;
 
-  if (!create_info->default_table_charset)
-  {
-    create_info->default_table_charset= system_charset_info;
-  }
+  set_table_default_charset(session, create_info, db);
 
   if (session->variables.old_alter_table
       || (table->s->db_type() != create_info->db_type)

